var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Module description files</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Module description files</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"module-description-files\"></a></div>\n\
<p>Modules provide a way to distribute multiple versions of a plug-in and load a different version depending on operating system, locale, and Maya version.</p>\n\
<p>Each entry in the module description file specifies conditions for loading specific versions of the plug-in and its supporting files, the location of the plug-in and its supporting files, and any required modifications to environment variables.</p>\n\
<p>You can comment out lines in the module description file by adding a <span class=\'code\'>#</span> to the beginning of the line.</p>\n\
<blockquote>\n\
<p><strong>Note:</strong> While module files should have a <span class=\'code\'>.mod</span> extension, all module files, regardless of extension, are read by Maya.</p>\n\
</blockquote>\n\
<p>The first line of every entry of a module description file provides the instructions for loading a specific version of the module for a specific platform and Maya version.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint lang-s\">+ MAYAVERSION:&lt;version&gt; PLATFORM:&lt;platform&gt; LOCALE:&lt;locale&gt; &lt;ModuleName&gt; &lt;ModuleVersion&gt; &lt;ModulePath&gt;\n\
</pre></div><p>| Entry | Description | Notes | | - | - | - | | <span class=\'code\'>MAYAVERSION</span> | Optional <br> Specifies a base version of Maya. | Can be any base version of Maya, for example 2023. <br> If this condition is omitted, the entry applies to all versions of Maya. | | <span class=\'code\'>PLATFORM</span> | Optional <br> Specifies an operating system. | Can be <span class=\'code\'>win64</span>, <span class=\'code\'>mac</span>, or <span class=\'code\'>linux</span>. <br> If this condition is omitted, the entry applies to all operating systems. | | <span class=\'code\'>LOCALE</span> | Optional <br> Specifies the system locale. | Can be <span class=\'code\'>en_US</span>, for US English, <span class=\'code\'>ja_JP</span>, for Japanese, or <span class=\'code\'>zh_CN</span>, for Simplified Chinese. <br> If this condition is omitted, the line applies to any locale. | | ModuleName | Required <br> The name of the module | | | ModuleVersion | Required <br> The module version. | | | ModulePath | Required <br> The module location. | Can be either an absolute path or a path relative to the module file. |</p>\n\
<blockquote>\n\
<p><strong>Note:</strong></p>\n\
<ul>\n\
<li>There must be a space between the leading <span class=\'code\'>+</span> and the first term of the first line.</li>\n\
<li>The module file will be moved to a location in <a href=\'#!/url=./dev_help/Distributing-Maya-Plug-ins/DistributingUsingModules/InstallingModules.html\' title=\'\'>the module search path when the module is installed</a>. You must take this into consideration when setting the paths to the module directories.</li>\n\
<li>If there are multiple entries for the same module that are identical except for their version number, Maya will only take into account the entry with the highest version number. Entries with lower version numbers will be ignored.</li>\n\
</ul>\n\
</blockquote>\n\
<p>Each entry specifies the location of the module files associated with the conditions in the first line.</p>\n\
<p>The paths to the files can be specified either as absolute paths or as paths relative to the <span class=\'code\'>.mod</span> file.</p>\n\
<p>The default structure of the module directory can be overwritten if files in a different directory need to be loaded, or if the directories need to be read recursively.</p>\n\
<p>Override the default location of a folder using:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint lang-s\">&lt;folder to override&gt;: &lt;new location for this folder&gt;\n\
</pre></div><p>For example, to specify a new location for Python and MEL scripts, add the line:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint lang-s\">scripts: ExtraFiles\\2023\\scripts\n\
</pre></div><p>To read the directory recursively, add <span class=\'code\'>[r]</span> in front of the directory:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint lang-s\">[r] scripts: ExtraFiles\\2023\\scripts\n\
<blockquote>\n\
</blockquote></pre></div><p><strong>Note:</strong> Hidden subdirectories will be ignored even when recursive search is on, as will any files or directories with the following extensions: * <span class=\'code\'>.so</span> * <span class=\'code\'>.mll</span> * <span class=\'code\'>.bundle</span> * <span class=\'code\'>.py</span> * <span class=\'code\'>.dll</span> * <span class=\'code\'>.framework</span> * <span class=\'code\'>.plugin</span> * <span class=\'code\'>.plist</span> * <span class=\'code\'>.app</span> * <span class=\'code\'>.pkg</span> * <span class=\'code\'>.rtfd</span> * <span class=\'code\'>.download</span> * <span class=\'code\'>.sparsebundle</span></p>\n\
\n\
<p>You can set and modify environment variables as needed when a specific plug-in is loaded.</p>\n\
<p>To set a variable, use the <span class=\'code\'>=</span> operator:</p>\n\
<p><span class=\'code\'>MY_VAR=&lt;value&gt;</span></p>\n\
<p>To set a path relative to the module directory, use the <span class=\'code\'>:=</span> operator.</p>\n\
<p>For example, the following entry sets the <span class=\'code\'>PYTHONPATH</span> to <span class=\'code\'>..\\CustomModule\\python</span>:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint lang-s\">+ MAYAVERSION:2023 PLATFORM:win64 CustomModule 1.2 ..\\CustomModule\n\
scripts: ExtraFiles\\2023\\scripts\n\
PYTHONPATH:=python\n\
</pre></div><p>You can append to the current value of an environment variable using the <span class=\'code\'>+=</span> operator.</p>\n\
<p>For example, the following appends <span class=\'code\'>C:\\ModuleFiles\\python</span> to the existing <span class=\'code\'>PYTHONPATH</span>:</p>\n\
<p><span class=\'code\'>PYTHONPATH+=C:\\ModuleFiles\\python</span></p>\n\
<p>To add a path relative to the module directory, use <span class=\'code\'>+:=</span>.</p>\n\
<p>For example, the following appends the <span class=\'code\'>python</span> directory within <span class=\'code\'>CustomModule</span> to the existing <span class=\'code\'>PYTHONPATH</span>:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint lang-s\">+ MAYAVERSION:2023 PLATFORM:x64 CustomModule 1.2 ..\\CustomModule\n\
scripts: ExtraFiles\\2023\\scripts\n\
PYTHONPATH+:=python\n\
</pre></div><p>Similarly, you can prepend paths to a variable using the <span class=\'code\'>*=</span> and <span class=\'code\'>*:=</span> operators.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";