var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Manipulator Containers</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Manipulator Containers</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"manipulator-containers\"></a></div>\n\
<p>You can create a manipulator container programmatically and add one or more base manipulators to the container. This approach involves the following steps:</p>\n\
<ol>\n\
<li>Create a manipulator container class derived from <span class=\'code\'>MPxManipContainer</span>.</li>\n\
<li>Add the base manipulators to the manipulator container class.</li>\n\
<li>Define associations between the manipulator and the attribute on the nodes.</li>\n\
</ol>\n\
<p>The parent class of user-defined container manipulators is <span class=\'code\'>MPxManipContainer</span>. There are a number of methods in <span class=\'code\'>MPxManipContainer</span> that allow you to add a variety of base manipulators to the container. You also need to implement a number of methods in the custom manipulator.</p>\n\
<p>The necessary methods are:</p>\n\
<ul>\n\
<li><a href=\'#!/url=./dev_help/Manipulators/Manipulator-Containers.html#creator-method\' title=\'\'>creator method</a></li>\n\
<li><a href=\'#!/url=./dev_help/Manipulators/Manipulator-Containers.html#initialize-method\' title=\'\'>initialize method</a></li>\n\
<li><a href=\'#!/url=./dev_help/Manipulators/Manipulator-Containers.html#createchildren-method\' title=\'\'>createChildren method</a></li>\n\
<li><a href=\'#!/url=./dev_help/Manipulators/Manipulator-Containers.html#connecttodependnode-method\' title=\'\'>connectToDependNode method</a>.</li>\n\
</ul>\n\
<p>You can also override the draw method to customize the way your container manipulator is drawn.</p>\n\
<div class=\'section\'><a id=\"creator-method\"></a><h2 id=\"creator-method\">creator method</h2></div>\n\
<p>The creator method needs to return a new instance of the manipulator, and it is registered in the <span class=\'code\'>initializePlugin()</span> function with a call to the <span class=\'code\'>MFnPlugin::registerNode()</span> method.</p>\n\
<p>Note: Because this method is static and registered, and not derived, the name of the method does not need to be <em>creator</em>, although by convention the name <em>creator</em> is typically used.</p>\n\
<div class=\'section\'><a id=\"initialize-method\"></a><h2 id=\"initialize-method\">initialize method</h2></div>\n\
<p>The initialize method performs any necessary initializations for the manipulator as well as calling the method in the parent class <span class=\'code\'>MPxManipContainer::initialize()</span>. Similar to the creator method, the initialize method is static and registered rather than derived.</p>\n\
<div class=\'section\'><a id=\"createchildren-method\"></a><h2 id=\"createchildren-method\">createChildren method</h2></div>\n\
<p>The base manipulators are added to the manipulator container classes using the <span class=\'code\'>MPxManipContainer::createChildren()</span> method.</p>\n\
<p>For example, in the <span class=\'code\'>moveManip::createChildren()</span> method:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus moveManip::createChildren()\n\
{\n\
    ...\n\
\n\
    fDistanceManip = addDistanceManip(manipName,\n\
    distanceName); \n\
    fFreePointManip = addFreePointTriadManip(pointManipName,\n\
    pointName);\n\
\n\
    ...\n\
\n\
}\n\
\n\
</pre></div><p>This method is usually called after the custom manipulator container class is set up. The <span class=\'code\'>MPxManipContainer</span> class provides a set of member functions to add individual base manipulators, most of them are named as addXYZManip, where XYZ represents the manipulator name. The function returns an <span class=\'code\'>MDagPath</span> object representing the created base manipulator. Following is an example from the footPrintManip plug-in for adding distance manipulator to a custom manipulator container.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus footPrintLocatorManip::createChildren() {\n\
    MStatus stat =MStatus::kSuccess;\n\
 \n\
    MString manipName(&quot;distanceManip&quot;);\n\
    MString distanceName(&quot;distance&quot;);\n\
 \n\
    MPoint startPoint (0.0, 0.0, 0.0);\n\
    MVector direction (0.0, 1.0, 0.0);\n\
    fDistanceManip =addDistanceManip(manipName,distanceName);\n\
 \n\
    MFnDistanceManip distanceManipFn(fDistanceManip);\n\
    distanceManipFn.setStartPoint(startPoint);\n\
    distanceManipFn.setDirection(direction);\n\
     \n\
    return(stat);\n\
}\n\
</pre></div><div class=\'section\'><a id=\"connecttodependnode-method\"></a><h2 id=\"connecttodependnode-method\">connectToDependNode method</h2></div>\n\
<p>The <span class=\'code\'>connectToDependNode()</span> method is where the association is made between the manipulator and the plug with which it will communicate. All the operations to set up communication between manipulators and plugs (including one-on-one association and conversion functions) must be put in this method. After mapping relationships between manipulators and plugs are set, there are two additional methods to be called. These methods are <span class=\'code\'>MPxManipContainer::finishAddingManips()</span> and <span class=\'code\'>MPxManipContainer::connectToDependNode()</span>, and must be called in that order. It is important to note that <span class=\'code\'>MPxManipContainer::finishAddingManips()</span> must be called after all calls to connect to plugs have been made. Furthermore, <span class=\'code\'>finishAddingManips()</span> must be called only once.</p>\n\
<p>For example, in the <span class=\'code\'>moveManip::connectToDependNode()</span> method:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus moveManip::connectToDependNode(const MObject &amp;node)\n\
{\n\
    ...\n\
    distanceManipFn.connectToDistancePlug(syPlug);\n\
    ...\n\
    freePointTriadManipFn.connectToPointPlug(tPlug);\n\
    ...\n\
    finishAddingManips();\n\
    ...\n\
    MPxManipContainer::connectToDependNode(node);\n\
    ...\n\
}\n\
\n\
</pre></div><p>Note: Although <span class=\'code\'>connectToDependNode()</span> is a virtual method, you will usually be responsible for calling this method yourself when you are using the manipulator within a custom context. For more information, see <a href=\'#!/url=./dev_help/Manipulators/Connect-Manipulators-to-the-Show.html\' title=\'\'>Connect Manipulators to the Show Manipulator Tool</a>.</p>\n\
<p>The footPrintManip plug-in example demonstrates the following operations: creating a distance manipulator, applying <span class=\'code\'>MFnDistanceManip</span> function set class on the created distance manipulator, setting up one-on-one association between plug and distance manipulator value, and setting up plugToManip conversion to set the manipulator start point position.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus footPrintLocatorManip::connectToDependNode (const MObject &amp;node) {\n\
    MStatus stat;\n\
 \n\
    // Get the DAG path\n\
    MFnDagNode dagNodeFn(node);\n\
    dagNodeFn.getPath(fNodePath);\n\
 \n\
    // Connect the plugs\n\
    \n\
    MFnDistanceManip distanceManipFn(fDistanceManip);\n\
    MFnDependencyNode nodeFn(node);\n\
 \n\
    MPlug sizePlug =nodeFn.findPlug(&quot;size&quot;, &amp;stat);\n\
    if ( stat != MStatus::kFailure ) {\n\
       distanceManipFn.connectToDistancePlug (sizePlug);\n\
       unsigned startPointIndex =distanceManipFn.startPointIndex();\n\
       addPlugToManipConversionCallback (startPointIndex,\n\
       (plugToManipConversionCallback)&amp;footPrintLocatorManip::startPointCallback);\n\
       finishAddingManips();\n\
       MPxManipContainer::connectToDependNode(node);\n\
    }\n\
    return(stat);\n\
}\n\
</pre></div><div class=\'section\'><a id=\"draw-method\"></a><h2 id=\"draw-method\">draw method</h2></div>\n\
<p>The draw method is an optional method which can be used to customize the drawing of a container manipulator. If you are overriding the draw method, you must first call <span class=\'code\'>MPxManipContainer::draw()</span> to draw all the children.</p>\n\
<p>For example, the <span class=\'code\'>moveManip::draw()</span> method draws a label in addition to the base manipulators.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">void moveManip::draw(M3dView &amp;view,\n\
                     const MDagPath &amp;path,\n\
                     M3dView::DisplayStyle style,\n\
                     M3dView::DispalyStatus status)\n\
{\n\
    MPxManipContainer::draw(view, path, style, status);\n\
    view.beginGL();\n\
    MPoint textPos(0, 0, 0);\n\
    char str[100];\n\
    sprintf(str, &quot;Stretch Me!&quot;);\n\
    MString distanceText(str);\n\
    view.drawText(distanceText, textPos, M3dView::kLeft);\n\
    view.endGL();\n\
}\n\
\n\
</pre></div><p>The shape and drawing of custom manipulators is restricted to base manipulators because the <span class=\'code\'>MPxManipContainer::draw()</span> method only provides a <span class=\'code\'>M3dView</span> parameter to add supplementary drawing to the manipulator. The custom manipulator’s shape is already decided by the combination of base manipulator shapes that are added to the current manipulator container. However, there are situations where you want to have more control over different perspectives of a manipulator. The <span class=\'code\'>MPxManipulatorNode</span> class, which was introduced in Maya 2009, provides a new way to implement custom manipulators with custom OpenGL drawing code. It offers options for selecting components (activating different handles) in custom manipulators. This class can either work alone or work with the <span class=\'code\'>MPxManipContainer</span> class.</p>\n\
<p>The <span class=\'code\'>MPxManipulatorNode::draw()</span> method is overridden to implement custom drawing. In OpenGL, drawing and picking are done together. There are several important methods like the following:</p>\n\
<ul>\n\
<li><span class=\'code\'>glFirstHandle()</span> – When drawing an OpenGL pickable component, a name uniquely representing the OpenGL component must be set. This method returns the unsigned integer value that represents the first available OpenGL handle to use. When one OpenGL component drawing is finished, the integer value can be incremented by one to represent the next OpenGL component.</li>\n\
<li><span class=\'code\'>colorAndName()</span> – This method is used to set the color of the OpenGL component that is being drawn next. It is also used to set the OpenGL name of the component so that picking is supported.</li>\n\
</ul>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">void triadScaleManip::draw (M3dView &amp;view, const MDagPath &amp;path, M3dView::DisplayStyle style,\n\
         M3dView::DisplayStatus status) {\n\
    ……\n\
 \n\
    // Begin the drawing\n\
    view.beginGL();\n\
 \n\
    // Place before you draw the manipulator component that can\n\
    // be pickable.\n\
    MGLuint glPickableItem;\n\
    glFirstHandle(glPickableItem);\n\
 \n\
    // Top\n\
    topName =glPickableItem;\n\
    colorAndName(view, glPickableItem, true, mainColor());\n\
    gGLFT-&gt;glBegin(GL_LINES);\n\
    gGLFT-&gt;glVertex3fv(tl);\n\
    gGLFT-&gt;glVertex3fv(tr);\n\
    gGLFT-&gt;glEnd();\n\
 \n\
    // Right\n\
    glPickableItem++;\n\
    rightName =glPickableItem;\n\
    colorAndName(view, glPickableItem, true, mainColor());\n\
    gGLFT-&gt;glBegin(GL_LINES);\n\
    gGLFT-&gt;glVertex3fv(tr);\n\
    gGLFT-&gt;glVertex3fv(br);\n\
    gGLFT-&gt;glEnd();\n\
 \n\
    // ...\n\
 \n\
    // End the drawing\n\
    view.endGL();\n\
}\n\
</pre></div><div class=\'section\'><a id=\"updating-plug-values-on-nodes\"></a><h2 id=\"updating-plug-values-on-nodes\">Updating Plug Values on Nodes</h2></div>\n\
<p>You can use <span class=\'code\'>MPxManipulatorNode</span> to update the plug values in the following ways:</p>\n\
<ul>\n\
<li><strong>Implement Methods Related to Mouse Movement Event</strong> – There are three methods for mouse movement events: <span class=\'code\'>MPxManipulatorNode::doPress()</span>, <span class=\'code\'>MPxManipulatorNode::doDrag()</span>, and <span class=\'code\'>MPxManipulatorNode::doRelease()</span>. These methods are called when the manipulator receives a corresponding mouse event such as mouse down, mouse drag, or mouse release. Algorithms can be implemented to update the plug values when these methods are called. For example, when <span class=\'code\'>doPress()</span> is called, you can record the mouse position as the original position or when <span class=\'code\'>doDrag()</span> is called, you can record the mouse movement (direction and distance).</li>\n\
<li><p><strong>Connect to a Dependent Node</strong> – Follow these steps to connect to a dependent node.</p>\n\
<ol>\n\
<li>Call <span class=\'code\'>add*Value()</span> in the <span class=\'code\'>postConstructor()</span> of the manipulator node.</li>\n\
<li>Call <span class=\'code\'>conectPlugToValue()</span> in <span class=\'code\'>connectToDependNode()</span>.</li>\n\
<li>Call <span class=\'code\'>set*Value()</span> in one of the <span class=\'code\'>do*()</span> functions.</li>\n\
</ol>\n\
<p>To connect plugs directly to manipulator values, the manipulator values must be first created on the manipulator. There are several <span class=\'code\'>add*Value()</span> methods to add manipulator values of different types. The one-on-one association can be achieved by calling <span class=\'code\'>connectPlugToValue()</span> method within <span class=\'code\'>MPxManipulatorNode::connectToDependNode()</span>. In one of the <span class=\'code\'>do*()</span> functions, which is triggered when the manipulator is receiving the corresponding mouse event, calling <span class=\'code\'>set*Value()</span> sets the corresponding manipulator value that consequently sets the plug value.</p></li>\n\
</ul>\n\
<div class=\'section\'><a id=\"registrationderegistration\"></a><h2 id=\"registrationderegistration\">Registration/Deregistration</h2></div>\n\
<p>Because manipulator containers are derived from nodes, user-defined manipulators can be registered and deregistered like any other node, with the exception that the <span class=\'code\'>MPxNode::Type</span> argument in <span class=\'code\'>MFnPlugin::registerNode()</span> is set to <span class=\'code\'>MPxNode::kManipContainer</span> instead of the default <span class=\'code\'>MPxNode::kDependNode</span>.</p>\n\
<p>For example, in moveToolManip.cpp:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus initializePlugin(MObject obj)\n\
{\n\
        ...\n\
        plugin.registerNode(&quot;moveManip&quot;, \n\
        moveManip::id,\n\
        &amp;moveManip::creator,\n\
        &amp;moveManip::initialize,\n\
        MPxNode::kManipContainer);\n\
        ...\n\
}\n\
MStatus uninitializePlugin(MObject obj)\n\
{\n\
    ...\n\
    plugin.deregisterNode(moveManip::id);\n\
    ...\n\
}\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";