var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Attributes and plugs</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Attributes and plugs</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"attributes-and-plugs\"></a></div>\n\
<p>Dependency nodes have attributes and plugs.</p>\n\
<p>Plugs are a means to pass input and output between nodes. Attributes are</p>\n\
<p>The node attributes added with <span class=\'code\'>MPxNode.addAttribute()</span> in <span class=\'code\'>nodeInitializer()</span> are used by Maya to define the input and output connection points of our node when it is instantiated in the Dependency Graph.</p>\n\
<p>These connection points are known as <em>plugs</em>, and are represented by instances of <span class=\'code\'>MPlug</span>. The call to <span class=\'code\'>MPxNode.attributeAffects()</span> in <span class=\'code\'>nodeInitializer()</span> determines the causal input &gt; output link between these plugs. Ultimately, the Dependency Graph keeps track of the directed edges of connected plugs among all the nodes. During Maya&#39;s ongoing Dependency Graph evaluation process, if data associated to a plug is changed, this plug is marked as <em>dirty</em>, meaning that its cached value is stale and must be recomputed. The plug hierarchy is recursively traversed to mark the remaining affected plugs as dirty.</p>\n\
<p>Maya then calls the <span class=\'code\'>compute()</span> method on each node containing a dirty plug, passing an instance of <span class=\'code\'>MPlug</span> as well as <span class=\'code\'>MDataBlock</span>. The related attribute of the passed <span class=\'code\'>MPlug</span> should be checked to determine what to compute. This is done using <span class=\'code\'>MPlug</span>&#39;s overloaded equality operator &#39;<span class=\'code\'>==</span>&#39;.</p>\n\
<p>The passed <span class=\'code\'>MDataBlock</span> instance contains <span class=\'code\'>MDataHandle</span> objects, which store the values associated to their respective attributes. The <span class=\'code\'>float</span> input value is obtained as follows:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">sampleInDataHandle = pDataBlock.inputValue( myNode.sampleInAttribute )\n\
sampleInValue = sampleInDataHandle.asFloat()\n\
</pre></div><p>The <span class=\'code\'>compute()</span> method should conclude by setting a newly computed value to the correct output <span class=\'code\'>MDataHandle</span>. The call to <span class=\'code\'>MDataHandle.setClean()</span> removes the dirty flag from its corresponding plug.</p>\n\
<p>If the plug passed into the <span class=\'code\'>compute()</span> function is not related to an output attribute we can compute, the function should return <span class=\'code\'>OpenMaya.kUnknownParameter</span>. This lets Maya know that the plug&#39;s value should be computed using one of our base class&#39;s <span class=\'code\'>compute()</span> functions.</p>\n\
<p>A custom node is defined by creating a Maya Dependency Graph plug-in. Such a plug-in contains a class which inherits from <span class=\'code\'>MPxNode</span> or one of its subclasses. This new class will override the <span class=\'code\'>MPxNode.compute()</span> function to define the custom node&#39;s behavior. Once a Dependency Graph plug-in is loaded into Maya, the Script Editor can be used to create an instance of the node.</p>\n\
<div class=\'figure\'><img src=\'images/MayaAttributevsPlug_75%.png\' title=\'\'></div>\n\
<p>In the previous diagram, the <span class=\'code\'>initializePlugin()</span> function makes a call to <span class=\'code\'>mplugin.registerNode()</span>, passing the parameter &quot;<span class=\'code\'>mynode</span>&quot; to declare the custom node type&#39;s name. An instance of this node can therefore be created using the following Python script:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">import maya.cmds as cmds\n\
cmds.createNode( &quot;mynode&quot; )\n\
</pre></div><p>The result is an instance of a node whose input and output <span class=\'code\'>MPlug</span>s are initialized according to the attributes declared in the plug-in (<span class=\'code\'>nodeInitializer()</span>). More concretely, the attributes declared in the plug-in will dictate how the <span class=\'code\'>MPlug</span>s will be created in terms of their type, default value, visibility in the UI, readability (usable as input), writeability (usable as output), and whether or not the run-time value should be stored to a file when the scene is saved.</p>\n\
<p>Observe that in the <span class=\'code\'>nodeInitializer()</span> function of our plug-in definition, we declare an <em>affects</em> relationship between:</p>\n\
<ul>\n\
<li><span class=\'code\'>inputAttribute1</span> and <span class=\'code\'>outputAttribute</span>,</li>\n\
<li><span class=\'code\'>inputAttribute2</span> and <span class=\'code\'>outputAttribute</span>.</li>\n\
</ul>\n\
<p>The <em>affects</em> relationship, declared using <span class=\'code\'>MPxNode.attributeAffects()</span>, tells Maya which inputs are required to compute a given output. As such, for any given instance of our node, if the <span class=\'code\'>MPlug</span> associated to its <span class=\'code\'>inputAttribute1</span> is marked dirty, the dirty flag will also propagate to the <span class=\'code\'>MPlug</span> associated to its <span class=\'code\'>outputAttribute</span>. This dirty flag propagation will therefore cause the node&#39;s <span class=\'code\'>compute()</span> function to be called to clean the dirty <span class=\'code\'>outputAttribute</span> <span class=\'code\'>MPlug</span>.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";