var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>MSelectionList</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>MSelectionList</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"mselectionlist\"></a></div>\n\
<p><span class=\'code\'>MSelectionList</span> provides you with methods to add and remove objects from the selection list, as well as walk through the objects on the list.</p>\n\
<p>For example, the following simple plug-in prints the names of all selected DAG nodes. If you create geometry and then select it, this plug-in prints the name of each selected object.</p>\n\
<div class=\'section\'><a id=\"simple-plug-in-example\"></a><h2 id=\"simple-plug-in-example\">Simple plug-in example</h2></div>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">#include &lt;maya/MSimple.h&gt;\n\
#include &lt;maya/MGlobal.h&gt;\n\
#include &lt;maya/MString.h&gt;\n\
#include &lt;maya/MDagPath.h&gt;\n\
#include &lt;maya/MFnDagNode.h&gt;\n\
#include &lt;maya/MSelectionList.h&gt;\n\
#include &lt;maya/MIOStream.h&gt;\n\
MStatus pickExample::doIt( const MArgList&amp; )\n\
{\n\
    MDagPath node;\n\
    MObject component;\n\
    MSelectionList list;\n\
    MFnDagNode nodeFn;\n\
    MGlobal::getActiveSelectionList( list );\n\
    for ( unsigned int index = 0; index &lt; list.length(); index++ )\n\
    {\n\
        list.getDagPath( index, node, component );\n\
        nodeFn.setObject( node );\n\
        cout &lt;&lt; nodeFn.name().asChar() &lt;&lt; &quot; is selected&quot; &lt;&lt; endl;\n\
    }\n\
    return MS::kSuccess;\n\
}\n\
DeclareSimpleCommand( pickExample, &quot;Autodesk&quot;, &quot;1.0&quot; );\n\
\n\
</pre></div><p>Walking through the list of selected objects is quite simple. If you create geometry and then select it, this plug-in prints the name of each selected object.</p>\n\
<p>The <span class=\'code\'>setObject()</span> method on <span class=\'code\'>MFnDagNode</span> is inherited by all function sets from <span class=\'code\'>MFnBase</span> and is used to set the object the function set will operate on. Usually this is done through the function set&#39;s constructor, but if the function set has already been created and you want to change the object the function set operates on, you use <span class=\'code\'>setObject()</span>. This is far more efficient than creating and destroying function sets each time you need one.</p>\n\
<p>Try selecting a few CVs and then calling this plug-in. Notice that you do not get the name of the CV output, but rather the name of the parent object (the curve, surface, or mesh). Also notice that the number of objects selected is not the same as the number of names printed.</p>\n\
<p>The Maya selection architecture simplifies the selection of object components such as CVs. Rather than putting each component object (for instance, each CV) onto the selection list, the parent object is put on the list and the components are grouped together.</p>\n\
<p>For example, if several CVs of <span class=\'code\'>nurbSphereShape1</span> were selected, the <span class=\'code\'>list.getDagPath()</span> call above would return an <span class=\'code\'>MDagPath</span> to <span class=\'code\'>nurbSphereShape1</span> and an <span class=\'code\'>MObject</span> grouping all the selected CVs together. The <span class=\'code\'>MDagPath</span> and the <span class=\'code\'>MObject</span> could then be passed to an <span class=\'code\'>MItSurfaceCV</span> iterator to examine the selected CVs.</p>\n\
<p>As long as you continue to select components of only one object, the object appears on the selection list only once. However, if you pick some components of one object, and then some components of another object, and then more components of the first object, the first object would actually appear on the selection list twice. This is so that you can determine the order in which objects were selected.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";