var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Writing a shape</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Writing a shape</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"writing-a-shape\"></a></div>\n\
<p>The goal of writing a shape is to take a geometry class you defined and integrate that geometry into Maya in the form of a DAG object.</p>\n\
<p>It is a good idea to keep geometry-specific information in your geometry class. This way it will be easier to represent the geometry as dependency graph data. Keeping geometry specific information in your geometry class will also make it easier to handle drawing as you will have to pass geometry-specific drawing information onto Maya’s draw request queue as draw data.</p>\n\
<div class=\'section\'><a id=\"where-to-start\"></a><h2 id=\"where-to-start\">Where to start</h2></div>\n\
<p>User-defined shapes are more complex than DG nodes because of the additional drawing, selection, and component functionality. It is a good idea to start by designing the shape node before writing any code. This involves defining your input and output attributes and determining the relationships (or affects) between those attributes.</p>\n\
<p>The simplest shape is one without components or geometry data—for example, a sphere shape that takes an input radius and perhaps x and y divisions and from this computes the geometry for a sphere to draw in OpenGL.</p>\n\
<p>It is also a good idea to write your shape in stages. The first step is to derive from <span class=\'code\'>MPxSurfaceShape</span> and <span class=\'code\'>MPxSurfaceShapeUI</span>. Writing the <span class=\'code\'>MPxSurfaceShape</span> class is the same as writing any <span class=\'code\'>MPxNode</span> class. The additional virtual functions can be overridden as the functionality is needed.</p>\n\
<div class=\'section\'><a id=\"registering-and-deregistering-shapes\"></a><h2 id=\"registering-and-deregistering-shapes\">Registering and deregistering shapes</h2></div>\n\
<p>Registering shapes with Maya is similar to registering DG nodes. The only difference is that shapes have a separate UI class that must be registered with the shape node. The <span class=\'code\'>MFnPlugin::resgisterShape</span> function is used for shape registration. Deregistering a shape is exactly the same as deregistering a node. Here is an example of shape registration:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus initializePlugin( MObject obj )\n\
{\n\
    MFnPlugin plugin( obj,&quot;Autodesk&quot;,&quot;2.0&quot;, &quot;Any&quot;);\n\
    return plugin.registerShape( &quot;yourShape&quot;, yourShape::id,\n\
        &amp;yourShape::creator,\n\
        &amp;yourShape::initialize,\n\
        &amp;yourShapeUI::creator );\n\
}\n\
\n\
MStatus uninitializePlugin( MObject obj )\n\
{\n\
    return plugin.deregisterNode( yourShape::id );\n\
}\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";