var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Internationalization Process</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Internationalization Process</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"internationalization-process\"></a></div>\n\
<p>Internationalization enables the plug-in to support localized strings in its user interface and to operate correctly in localized environments. The process of localizing the string resources is a separate set of steps outlined later in this document. Localization can take place any time after the plug-in has been internationalized, the plug-in will continue to work using the default resource values until translations are available.</p>\n\
<p>The following steps are involved in internationalization:</p>\n\
<ol>\n\
<li>Determine what aspects of the plug-in will require localization.</li>\n\
<li>Identify the user interface strings that need to change when the plug-in is running in another language. The strings may be located in C++ code or MEL scripts. Modify the code and scripts to use string resources in place of hard-coded strings.</li>\n\
<li>Add string registration calls to the plug-in initialization sequence.</li>\n\
<li>Review string handling code and make modifications as required to properly handle strings in both single and multi-byte encodings.</li>\n\
</ol>\n\
<div class=\'section\'><a id=\"assessing-requirements\"></a><h2 id=\"assessing-requirements\">Assessing Requirements</h2></div>\n\
<p>Before doing the work to internationalize a plug-in, it is useful to assess what the actual requirements are. Not all aspects of plug-in internationalization may be necessary or appropriate for each situation.</p>\n\
<p>The most important consideration is to determine the number of target languages the plug-in needs to support. If a plug-in is only targeted at a single language (for example, only Japanese) it may be possible to supply the custom UI for the plug-in in a localized script instead of using string resources.</p>\n\
<p>String resources are required if the plug-in meets one or more of the following criteria:</p>\n\
<ul>\n\
<li>the plug-in is to support more than one user interface language (for example, English and Japanese)</li>\n\
<li>the plug-in issues user messages from C++ code (for example, calls to <span class=\'code\'>displayError()</span>)</li>\n\
<li>the plug-in creates nodes which are to be displayed in the UI with localized node and attribute name labels</li>\n\
</ul>\n\
<p>Another consideration is the type of string handling and I/O the plug-in code performs. Plug-ins with little or no string manipulation or file handling should require few changes, while others will need to be reviewed to ensure they are dealing properly with multi-byte characters.</p>\n\
<div class=\'section\'><a id=\"defining-and-registering-string-resources\"></a><h2 id=\"defining-and-registering-string-resources\">Defining and Registering String Resources</h2></div>\n\
<p>This section will discuss how string resources are used in C++ and MEL scripts, and how to register them during plug-in initialization.</p>\n\
<div class=\'section\'><a id=\"string-resource-keys\"></a><h3 id=\"string-resource-keys\">String Resource Keys</h3></div>\n\
<p>Each string resource in the Maya String Catalog is identified by a unique key. For plug-ins, the key consists of an ordered pair of strings. The first element of the pair is the plug-in name, which will be the same for all strings used by the plug-in. The second part of the key is a unique identifier for the string being defined. For example, string resource keys used by the <span class=\'code\'>closestPointOnCurve</span> plug-in will have the form:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">(&quot;closestPointOnCurve&quot;, &quot;stringId1&quot;);\n\
(&quot;closestPointOnCurve&quot;, &quot;stringId2&quot;);\n\
(&quot;closestPointOnCurve&quot;, &quot;stringId3&quot;);\n\
\n\
</pre></div><div class=\'section\'><a id=\"string-resources-in-c\"></a><h3 id=\"string-resources-in-c\">String Resources in C++</h3></div>\n\
<p>The <span class=\'code\'>MStringResourceId</span> and <span class=\'code\'>MStringResource</span> classes are used to define and access string resources in the plug-in C++ code. The <span class=\'code\'>MStringResourceId</span> constructor accepts three arguments, the two elements used to form the resource key, and the default value of the resource string. In the example below the plug-in name is &quot;<span class=\'code\'>closestPointOnCurve</span>&quot;, and the unique user-defined key being given to this string is &quot;<span class=\'code\'>kNoValidObject</span>&quot;.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStringResourceId invalidObject(&quot;closestPointOnCurve&quot;, &quot;kNoValidObject&quot;, &quot;A curve or its transform node must be specified as a command argument, or using your current selection.&quot;);\n\
\n\
</pre></div><p>Using #define statements to associate the <span class=\'code\'>MStringResourceId</span> with a constant is helpful to provide a single point of definition for each resource. Typically the <span class=\'code\'>MStringResourceId</span> declarations for the plug-in will be grouped together in a header file that is shared between the C++ modules that require it.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">#define kPluginId  &quot;closestPointOnCurve&quot;\n\
#define kNoValidObject MStringResourceId(kPluginId, &quot;kNoValidObject&quot;, \\&quot;A curve or its transform node must be specified as a command argument, or using your current selection.&quot;)\n\
\n\
</pre></div><p>The <span class=\'code\'>MStringResource::getString</span> method is used to look up the catalog entry when the string is needed in the code. The catalog lookup will return either the default value or localized string value (if it is available). Once the string is loaded it can be used like any other <span class=\'code\'>MString</span>.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus stat;\n\
MString msg = MStringResource::getString(kNoValidObject, stat);\n\
displayError(msg);\n\
\n\
</pre></div><p>A string resource cannot be accessed until it has been registered. This is done by calling <span class=\'code\'>MstringResource::registerString</span>. The registration steps are described below under String Resource Registration.</p>\n\
<div class=\'section\'><a id=\"string-resources-in-mel-scripts\"></a><h3 id=\"string-resources-in-mel-scripts\">String Resources in MEL Scripts</h3></div>\n\
<p>MEL scripts can use string resources in a similar manner to the C++ code.</p>\n\
<p>To use a string resource, the value is retrieved using the <span class=\'code\'>getPluginResource</span> command. The arguments passed are the two elements of the string resource key.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">string $titleStr = getPluginResource(&quot;closestPointOnCurve&quot;, &quot;kAETitle&quot;);\n\
editorTemplate -beginScrollLayout;\n\
editorTemplate -beginLayout $titleStr -collapse 0;\n\
\n\
</pre></div><p>MEL resources are registered using the <span class=\'code\'>registerPluginResource</span> command. The registration process is described below.</p>\n\
<div class=\'section\'><a id=\"string-resource-registration\"></a><h3 id=\"string-resource-registration\">String Resource Registration</h3></div>\n\
<p>This section describes the process of registering string resources in the plug-in. All string registration is done during plug-in initialization.</p>\n\
<p>Each string resource must be registered before it can be used. The registration step ensures that the resource&#39;s default value is loaded into the string catalog. After the default values are registered the plug-in writer must add a call which will load the localized resource values. When localized resource values are available for the language Maya is running in, the localized values will override the default values.</p>\n\
<p>The main registration method is <span class=\'code\'>MFnPlugin::registerUIStrings()</span>. A call to this routine is added to the plug-in&#39;s <span class=\'code\'>initializePlugin()</span> function. It should be placed early in the initialization sequence since the string resources will not be available until it is called, and some of the other initialization methods may require them.</p>\n\
<p>The <span class=\'code\'>MFnPlugin::registerUIStrings</span> function takes two arguments. The first argument is the name of a procedure which will register the strings used in the C++ code. The second argument is the name of a script that will register the string resources used in MEL scripts.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\"> // Register string resources used in the code and scripts\n\
status = plugin.registerUIStrings(registerMStringResources, &quot;closestPointOnCurveInitStrings&quot;);\n\
if (!status)\n\
{\n\
    status.perror(&quot;registerUIStrings&quot;);\n\
    return status;\n\
}\n\
\n\
</pre></div><p>The C++ routine <span class=\'code\'>registerMStringResources</span> referenced in this call registers each <span class=\'code\'>MStringResourceId</span> used by the C++ code.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">// Register all strings used by the plugin C++ source \n\
static MStatus registerMStringResources(void)\n\
{\n\
    MStringResource::registerString(kNoValidObject);\n\
    // other resources would go here\n\
    return MS::kSuccess;\n\
}\n\
\n\
</pre></div><p>This example is using a constant previously defined in a header file:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">#define kPluginId  &quot;closestPointOnCurve&quot;\n\
#define kNoValidObject MStringResourceId(kPluginId, &quot;kNoValidObject&quot;, \\&quot;A curve or its transform node must be specified as a command argument, or using your current selection.&quot;)\n\
\n\
</pre></div><p>The second argument is the name of a script which initializes all resources used by the plug-in&#39;s MEL scripts. Each resource is registered with a call to <span class=\'code\'>registerPluginResources</span>.</p>\n\
<p>The initialization script serves a dual purpose. In addition to the MEL resource registration it also contains the logic to load language-dependent resources for the plug-in. The routine <span class=\'code\'>loadPluginLanguageResources</span> takes the name of a resource file that will contain the localized version of the plug-in string resources. More details creating and installing the localized resource file are found under Localization Process below.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">global proc closestPointOnCurveInitStrings()\n\
{\n\
    // Register script resources\n\
    registerPluginResource(&quot;closestPointOnCurve&quot;, &quot;kAETitle&quot;,\n\
        &quot;Closest Point On Curve Attributes&quot;);\n\
    registerPluginResource(&quot;closestPointOnCurve&quot;, &quot;kInputCurve&quot;,\n\
        &quot;Input Curve&quot;);\n\
    registerPluginResource(&quot;closestPointOnCurve&quot;, &quot;kResults&quot;,\n\
        &quot;Results&quot;);\n\
    // Load any localized resources\n\
    loadPluginLanguageResources(&quot;closestPointOnCurve&quot;, &quot;closestPointOnCurve.pres.mel&quot;);\n\
}\n\
\n\
</pre></div><p>Once the registration sequence is complete the strings are available in the Maya Catalog and can be retrieved using <span class=\'code\'>MString::getStringResource</span> (C++) or <span class=\'code\'>getPluginResource</span> (MEL). If localized values for the resources were located by <span class=\'code\'>loadPluginLanguageResources</span> they will be returned from the catalog instead of the default values.</p>\n\
<div class=\'section\'><a id=\"string-handling\"></a><h2 id=\"string-handling\">String Handling</h2></div>\n\
<p>Plug-in writers using the <span class=\'code\'>MString</span> class are largely insulated from locale-dependent changes, but some string handling code may require changes to operate correctly in both single and multi-byte environments. This section mainly focuses on issues as they relate to the use of <span class=\'code\'>MString</span> in a localized environment but many of the problems described apply in to character handling in general.</p>\n\
<div class=\'section\'><a id=\"encoding\"></a><h3 id=\"encoding\">Encoding</h3></div>\n\
<p>The <span class=\'code\'>MString</span> class operates under the assumption that by default, character data in char* form is encoded in the codeset of the locale. This is a natural extension of the existing functionality of the class and in many cases an existing plug-in will continue to work without changes in a localized environment. New methods have been added to explicitly assign or access the string using UTF-8 and wide character formats which are commonly used in internationalized applications.</p>\n\
<div class=\'section\'><a id=\"string-length-and-position-values\"></a><h3 id=\"string-length-and-position-values\">String Length and Position Values</h3></div>\n\
<p>The most common problem when dealing with localized text is the correct interpretation of string length. In multi-byte environments, the character (<span class=\'code\'>char *</span>) representation of the string will use one or more bytes to represent each character in the string. This means that the string&#39;s storage length in bytes does not necessarily correspond to the number of individual characters in the string and code using this assumption may not behave as expected. For backwards compatibility, the <span class=\'code\'>MString::length</span> method will continue to return the number of bytes in the character buffer, a new method <span class=\'code\'>MString::numChars</span> can be used instead when it is necessary to determine the number of individual characters in the string.</p>\n\
<p>The interpretation of positional indexes into the string data is similarly problematic in a multi-byte environment (for example when using the <span class=\'code\'>MString::substring</span> method). See the <a href=\'#!/url=./dev_help/Technical-Notes/Internationalizing-plug-ins/Internationalization-Process.html#new-mstring-methods\' title=\'\'>New MString Methods</a> for details on what has been added to deal correctly with multi-byte strings.</p>\n\
<div class=\'section\'><a id=\"formatting-message-strings\"></a><h3 id=\"formatting-message-strings\">Formatting Message Strings</h3></div>\n\
<p>User message strings are often built by concatenating multiple strings or variables. This technique is not appropriate for strings that will be localized, since the context and placement of the strings may need to change for another language.</p>\n\
<p>The <span class=\'code\'>MString::format</span> method or MEL <span class=\'code\'>format</span> command should be used to format the string. Format allows positional arguments to be correctly placed in context when the string is translated.</p>\n\
<p>The following example shows an original block of code which creates an error message string containing the name of a file using string concatenation:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MString filename;\n\
MString msg;\n\
msg = &quot;The file &quot;;\n\
msg += filename;\n\
msg += &quot; cannot be opened&quot;;\n\
displayError(msg);\n\
\n\
</pre></div><p>The following replacement code shows how this would be done correctly in an internationalized application. The message is being created using a resource string and the <span class=\'code\'>MString::format</span> command:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">#define kOpenError MStringResourceId(&quot;myPlugin&quot;, &quot;kOpenError&quot;, &quot;File ^1s cannot be opened&quot;);\n\
MString filename;\n\
MString msgFmt = MStringResource::getString(kOpenError,status);\n\
MString msg;\n\
msg.format(msgFmt, filename);\n\
displayError(msg);\n\
\n\
</pre></div><p>MEL scripts can make use of the MEL format command in a similar manner.</p>\n\
<div class=\'section\'><a id=\"new-mstring-methods\"></a><h3 id=\"new-mstring-methods\">New MString Methods</h3></div>\n\
<p>The following table lists new methods that have been added to <span class=\'code\'>MString</span> to support internationalization. See the <span class=\'code\'>MString</span> class documentation for more details about each method, as well as notes about the behavior of existing methods within a localized environment.</p>\n\
<table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n\
<colgroup>\n\
<col width=\"50%\">\n\
<col width=\"50%\">\n\
</colgroup>\n\
<tbody>\n\
<tr class=\"ruled-heading\">\n\
<th class=\"table-heading\">\n\
New <span class=\"code\">MString</span> Method\n\
</th>\n\
<th class=\"table-heading\">\n\
Notes\n\
</th>\n\
</tr>\n\
<tr class=\"ruled-odd-row\">\n\
<td class=\"table-body\">\n\
<span class=\"code\">MString::numChars</span>\n\
</td>\n\
<td class=\"table-body\">\n\
This routine returns the number of characters in the string. This does not necessarily correspond to the number of bytes in the string, or the value returned by <span class=\"code\">MString::length</span>.\n\
</td>\n\
</tr>\n\
<tr class=\"ruled-even-row\">\n\
<td class=\"table-body\">\n\
<a name=\"GUID-E30E1E57-7963-489B-85CE-729D4E3B6722\"></a>\n\
<p class=\"table-body\">\n\
<span class=\"code\">MStatus MString::setUTF8( const char * utf8String )</span>\n\
</p>\n\
<a name=\"GUID-87511EB3-FA02-4981-A2CB-1DD630A15CCD\"></a>\n\
<p class=\"table-body\">\n\
<span class=\"code\">const char * MString::asUTF8()</span>\n\
</p>\n\
<a name=\"GUID-93692639-297D-4540-87CB-B707BAB67E61\"></a>\n\
<p class=\"table-body\">\n\
<span class=\"code\">const char * MString::asUTF8(int *length)</span>\n\
</p>\n\
</td>\n\
<td class=\"table-body\">\n\
These methods assign or access the string value as a UTF-8 encoded character string.\n\
</td>\n\
</tr>\n\
<tr class=\"ruled-odd-row\">\n\
<td class=\"table-body\">\n\
<a name=\"GUID-D3FDF0AC-26C1-4B04-B2B1-3F69092316F3\"></a>\n\
<p class=\"table-body\">\n\
<span class=\"code\">Mstring::MString(wchar_t *str)</span>\n\
</p>\n\
<a name=\"GUID-5B878F7B-B901-4FCF-9160-1398F33FFFCD\"></a>\n\
<p class=\"table-body\">\n\
<span class=\"code\">MString::MString(wchar_ *str, int length)</span>\n\
</p>\n\
<a name=\"GUID-002AB11F-ED63-42D0-94CE-5468852B0061\"></a>\n\
<p class=\"table-body\">\n\
<span class=\"code\">MStatus MString::setWChar(wchar_t *str, int length)</span>\n\
</p>\n\
<a name=\"GUID-187CF52C-C11B-4498-A615-DD330F23A931\"></a>\n\
<p class=\"table-body\">\n\
<span class=\"code\">MStatus MString::setWChar(wchar_t *str, int length)</span>\n\
</p>\n\
<a name=\"GUID-4E5C9D7F-F822-43D8-A2C5-5F53001B4EAD\"></a>\n\
<p class=\"table-body\">\n\
<span class=\"code\">const wchar_t* MString::asWChar()</span>\n\
</p>\n\
<a name=\"GUID-255E7BEA-6DC5-4A0B-A652-0B6A6644DA41\"></a>\n\
<p class=\"table-body\">\n\
<span class=\"code\">const wchar_t* MString::asWChar(int length)</span>\n\
</p>\n\
</td>\n\
<td class=\"table-body\">\n\
<p>These methods allow the string&#39;s value to be set or accessed using wide character values.</p>\n\
<blockquote>\n\
<strong>Note:</strong> the use of wide character representation is not recommended for persistent storage, use of a portable format such as UTF-8 is recommended instead.\n\
</blockquote></td>\n\
</tr>\n\
<tr class=\"ruled-even-row\">\n\
<td class=\"table-body\">\n\
<a name=\"GUID-E0DD2517-BCAC-4EDC-8682-58CC56CD9B7B\"></a>\n\
<p class=\"table-body\">\n\
<span class=\"code\">int MString::indexW(char c) const</span>\n\
</p>\n\
<a name=\"GUID-7BFA3217-4FC2-4106-9BD6-914B457CAE6F\"></a>\n\
<p class=\"table-body\">\n\
<span class=\"code\">int MString::indexW(wchar_t c) const</span>\n\
</p>\n\
<a name=\"GUID-DF0A7148-F737-42D6-9829-C824490FB9A4\"></a>\n\
<p class=\"table-body\">\n\
<span class=\"code\">int MString::rindexW(char c) const</span>\n\
</p>\n\
<a name=\"GUID-FD077AD4-9F6F-4406-8CFC-29443AF9177E\"></a>\n\
<p class=\"table-body\">\n\
<span class=\"code\">int MString::rindexW(wchar_t c) const</span>\n\
</p>\n\
</td>\n\
<td class=\"table-body\">\n\
These routines are multi-byte compatible versions of<span class=\"code\"> MString::index</span> and <span class=\"code\">MString::rindex</span> respectively and return character-based position values. Use of these routines for internationalized plug-ins is recommended. See also <span class=\"code\">MString::substringW</span>.\n\
</td>\n\
</tr>\n\
<tr class=\"ruled-odd-row\">\n\
<td class=\"table-body\">\n\
<span class=\"code\">MString MString::substringW(int start, int end)</span>\n\
</td>\n\
<td class=\"table-body\">\n\
This is a multi-byte compatible version of <span class=\"code\">MString::substring</span> which accepts character-based position values (such as those returned by <span class=\"code\">MString::indexW</span> and <span class=\"code\">MString::rindexW</span>).\n\
</td>\n\
</tr>\n\
<tr class=\"ruled-even-row\">\n\
<td class=\"table-body\">\n\
<span class=\"code\">MStatus MString::split(wchar_t c, MStringArray&amp; array) const</span>\n\
</td>\n\
<td class=\"table-body\">\n\
This version of <span class=\"code\">MString::split</span> accepts a wide-character value as the delimiter.\n\
</td>\n\
</tr>\n\
<tr class=\"ruled-odd-row\">\n\
<td class=\"table-body\">\n\
<span class=\"code\">MStatus MString::format( const MString &amp;fmt, const MString &amp;arg1, const MString &amp;arg2, ... ,const MString &amp;arg10 )</span>\n\
</td>\n\
<td class=\"table-body\">\n\
This routine provides string formatting capabilities, using a format specifier and up to 10 positional arguments.\n\
</td>\n\
</tr>\n\
</tbody>\n\
</table>\n\
\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";