var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Camera information in relation to DirectX and OpenGL draw APIs</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Camera information in relation to DirectX and OpenGL draw APIs</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"camera-information-in-relation-to-directx-and-opengl-draw-apis\"></a></div>\n\
<div class=\'section\'><a id=\"conventions\"></a><h2 id=\"conventions\">Conventions</h2></div>\n\
<p>DirectX and OpenGL have different camera conventions.</p>\n\
<ul>\n\
<li>DirectX uses a left-handed camera coordinate system, where the view direction points down the Z+ axis and the near and far clip planes are mapped to the [0,1] <em>normalized device coordinate</em> (NDC) range in the Z axis.</li>\n\
<li>OpenGL (and Maya) use a camera right-handed coordinate system. Additionally, the OpenGL view direction points down the Z- axis and the near and far clip planes map to the [-1,1] range in the Z axis.</li>\n\
</ul>\n\
<p>Viewport 2.0 remains consistent with the rest of Maya by using a right-handed coordinate system when setting the camera projection matrix. However, the near and far clip planes map to the [0,1] range in Z.</p>\n\
<p>Thus, when querying for:</p>\n\
<ul>\n\
<li>Depth range using <span class=\'code\'>MDrawContext::getDepthRange()</span>, the return value will be in the [0,1] range for DirectX and OpenGL.</li>\n\
<li>View direction using <span class=\'code\'>MDrawContext::viewDirectionAlongNegZ()</span>, the return value will always indicate that the view direction is along –Z for both OpenGL and DirectX.</li>\n\
</ul>\n\
<p>The world space camera vectors can be queried by passing the <span class=\'code\'>kViewDirection</span> (direction), <span class=\'code\'>kViewUp</span> (up), and <span class=\'code\'>kViewRight</span> (right) enums to the <span class=\'code\'>MFrameContext::getTuple()</span> method. These three vectors form the basis of a right-handed coordinate system.</p>\n\
<p><span class=\'code\'>MDrawContext</span> also provides access to the active camera. When querying its properties via the <span class=\'code\'>MFnCamera</span> interface, the same consistent camera vector values can be queried. If object space vectors are returned, then the values will show a consistent right-handed coordinate system with a Z- view direction.</p>\n\
<p>Some useful methods that can be used for querying include:</p>\n\
<ul>\n\
<li><span class=\'code\'>MFnCamera::unnormalizedNearClippingPlane()</span>. This returns values in object or world space, and not NDC.</li>\n\
<li><span class=\'code\'>MFnCamera::unnormalizedFarClippingPlane()</span>. This returns values in object or world space, and not NDC.</li>\n\
<li><span class=\'code\'>MFnCamera::eyePoint()</span>. If querying world space coordinates, this is equivalent to querying <span class=\'code\'>kViewPosition</span>.</li>\n\
<li><span class=\'code\'>MFnCamera::viewDirection()</span>. If querying world space coordinates, this is equivalent to querying <span class=\'code\'>kViewDirection</span>.</li>\n\
<li><span class=\'code\'>MFnCamera::upDirection()</span>. If querying world space coordinates, this is equivalent to querying <span class=\'code\'>kViewUp</span>.</li>\n\
<li><span class=\'code\'>MFnCamera::rightDirection()</span>. If querying world space coordinates, this is equivalent to querying <span class=\'code\'>kViewRight</span>.</li>\n\
</ul>\n\
<div class=\'section\'><a id=\"clipping-planes\"></a><h2 id=\"clipping-planes\">Clipping planes</h2></div>\n\
<p>With the Legacy Default Viewport (Viewport 1), it is possible to directly set supplemental clip planes in OpenGL when the active camera belongs to a camera set.</p>\n\
<p>With Viewport 2.0, it is not advisable to directly query the device, as fixed-function user clip planes may not be available for certain draw APIs. As such, the Viewport 1 behaviour is disabled by default for Viewport 2.0, though it can be accessed via an environment variable if needed when running OpenGL (ENABLE_DEFAULT_VIEWPORT_CAMERA_SETS).</p>\n\
<p>A draw API agnostic option to get world space clip planes is shown below. The camera is taken from the <span class=\'code\'>MDrawContext</span>, but can be for any camera shape. Note the use of the <em>unnormalized</em> methods to query clip planes.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MDrawContext context;\n\
\n\
// Get the current camera from the context\n\
MFnCamera activeCamera(context.getCurrentCameraPath());\n\
\n\
// Get relative near and far clip values with respect to the camera position\n\
// You should ignore any override values set when using a\n\
// camera set; therefore, do not use the nearClippingPlane() and farClippingPlane() \n\
// methods on MFnCamera.\n\
double nearD = activeCamera.unnormalizedNearClippingPlane();\n\
double farD = activeCamera.unnormalizedFarClippingPlane();\n\
\n\
// Get world space camera information\n\
MPoint eyePoint = activeCamera.eyePoint(MSpace::kWorld);\n\
MVector viewDirection = activeCamera.viewDirection(MSpace::kWorld); // Positive value\n\
\n\
double dist = eyePoint[0]*viewDirection[0] + \n\
              eyePoint[1]*viewDirection[1] + \n\
              eyePoint[2]*viewDirection[2]; \n\
\n\
// Compute near clip plane facing away from the camera direction (for OpenGL)\n\
double distNear = -1.0 * (dist + nearD); \n\
MVector OpenGL_NearClipPlaneVector(     \n\
       viewDirection[0], viewDirection[1], viewDirection[2], distNear);\n\
\n\
// Compute far clip plane facing towards the camera direction (for OpenGL)\n\
double distFar = dist + farD;\n\
MVector OpenGL_FarClipPlaneVector(\n\
       -viewDirection[0], -viewDirection[1], -viewDirection[2], distFar);\n\
</pre></div><p>The equivalent code if only using <span class=\'code\'>MDrawContext</span> would be as follows:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">// Query the camera coordinate system\n\
MDoubleArray vPos = context.getTuple(MHWRender::MFrameContext::kViewPosition);\n\
MDoubleArray vDir = context.getTuple(MHWRender::MFrameContext::kViewDirection);\n\
MDoubleArray vNear = context.getTuple(MHWRender::MFrameContext::kViewUnnormlizedNearClipValue);\n\
MDoubleArray vFar = context.getTuple(MHWRender::MFrameContext::kViewUnnormalizedFarClipValue);\n\
\n\
// Compute the world space planes for the near and far clip planes\n\
// The near plane is pointing away from the camera and is thus pointing in the\n\
// negative direction, while the far plane is pointing towards the camera (positive \n\
// direction.\n\
double distW = vPos[0]*vDir[0] + \n\
               vPos[1]*vDir[1] + \n\
               vPos[2]*vDir[2]; \n\
\n\
// Near clip plane faces away from the camera\n\
double distNearW = -1.0 * (distW + vNear[0]); \n\
MVector OpenGL_NearPlane(vDir[0], vDir[1], vDir[2], distNearW);\n\
\n\
// Far clip plane faces torwards the camera\n\
double distFarW = distW + vFar[0];\n\
MVector OpenGL_NearPlane(-vDir[0], -vDir[1], -vDir[2], distFarW);   \n\
</pre></div><div class=\'section\'><a id=\"depth-priority\"></a><h2 id=\"depth-priority\">Depth Priority</h2></div>\n\
<p>Shaders may use depth priority to offset the geometric position along the view direction. Within the shader code, the computation is relative to the actual device’s depth range, and thus differ for OpenGL versus DirectX. Assuming a uniform parameter called <span class=\'code\'>depthPriority</span> has been set appropriately, the following sample code performs the offset in DirectX and OpenGL:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">// DX range is [0,1] or 1 in size.\n\
float4 iPcPriority( float3 pm, float depthPriority, float4x4 worldViewProjectionC )\n\
{ \n\
    float4 P = mul( float4(pm,1), worldViewProjectionC ); \n\
    P.z -= P.w * depthPriority; \n\
    return P; \n\
}\n\
 \n\
// OpenGL range is [-1,1] or 2 in size\n\
vec4 iPcPriority( vec3 pm, float depthPriority, mat4 worldViewProjectionC )\n\
{\n\
    vec4 P = worldViewProjectionC * vec4(pm,1.0f);\n\
    P.z -= P.w * 2.0 * depthPriority;\n\
    return P;\n\
}\n\
</pre></div><div class=\'section\'><a id=\"selection\"></a><h2 id=\"selection\">Selection</h2></div>\n\
<p>Selection uses the same information as is used for drawing.</p>\n\
<div class=\'section\'><a id=\"screen-orientation\"></a><h2 id=\"screen-orientation\">Screen Orientation</h2></div>\n\
<p>It is worth noting that, for fat lines and points (generated using geometry shaders), or any screen aligned geometry, that a counter-clockwise winding order indicates front-facing geometry. This is consistent for both DirectX and OpenGL.</p>\n\
<p>For example, in the following DirectX code, the quad points are defined as follows:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">static const float4 cQuadPts[4] = {  \n\
    float4( -1.0,  1.0, 0, 0 ),  \n\
    float4( -1.0, -1.0, 0, 0 ),  \n\
    float4(  1.0,  1.0, 0, 0 ),  \n\
    float4(  1.0, -1.0, 0, 0 )};\n\
</pre></div><p>The geometry shader expansion traverses through the points in forward order, resulting in a triangle strip with the triangles each having a counter-clockwise orientation:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">void point2ScreenQuad( geometryInS inputs[1], float2 pointSz, float2 screenSize,\n\
     float4x4 viewProjInverse, float depthPriorityUnit, bool orthographic, float DPThresholdInView, \n\
     inout TriangleStream&lt;geometryInS&gt; outStream  ) \n\
{\n\
    geometryInS outS  = inputs[0]; \n\
    float size = max(0, max(pointSz.x, pointSz.y)); \n\
    float dpScale = orthographic ? 1.0f : -DPThresholdInView; \n\
    float dp = 2.0f * size * cDepthPriorityUnit * dpScale; \n\
    float4 sizeInZ = float4(pointSz.xy  / screenSize.xy, 0, 0) * outS.Pc.w;  \n\
    [unroll] for( int i = 0; i &lt; 4; ++i ) { \n\
        outS.Pc = inputs[0].Pc + sizeInZ * cQuadPts[i]; \n\
        outS.Pc.z = inputs[0].Pc.z - dp; \n\
        outStream.Append( outS ); \n\
    } \n\
    outStream.RestartStrip(); \n\
}\n\
</pre></div><div class=\'section\'><a id=\"examples\"></a><h2 id=\"examples\">Examples</h2></div>\n\
<ul>\n\
<li>The plug-in <em>rawFootPrintNode</em> in the Maya Developer Kit has example debug code that performs various <span class=\'code\'>MDrawContext</span>/<span class=\'code\'>MFrameContext</span> and <span class=\'code\'>MFnCamera</span> queries, including the world space clip plane computation described above.</li>\n\
<li>The <em>cgfxShader</em> plug-in shows an example of converting from normalized device coordinates (NDC) to screen space.</li>\n\
<li>The <span class=\'code\'>MayaBlinnDirectionalLightShadowGL.cgfx</span>, <span class=\'code\'>MayaBlinnDirectionalLightShadowGL.ogsfx</span>, and <span class=\'code\'>MayaBlinnDirectionalLightShadow10.fx</span> files provide DirectX and OpenGL sample implementations of using depth priority offsets for the same shader. See the <span class=\'code\'>bin\\Cg</span>, <span class=\'code\'>bin\\OGSFX</span> and <span class=\'code\'>bin\\HLSL</span> folders of the Maya installation directory for these files. The fragment <span class=\'code\'>mayaDepthPriorityShader</span> can also be examined as it is the <span class=\'code\'>iPcPriority</span> function discussed above.</li>\n\
<li>The <span class=\'code\'>mayaPoint2Quad</span> fragment can be examined for an example of geometry shader code used for drawing fat lines in the <span class=\'code\'>mayaFatPointShader</span>. It also incorporates the <span class=\'code\'>mayaDepthPriorityShader</span> fragment to allow view direction offsets.</li>\n\
</ul>\n\
<blockquote>\n\
<p><strong>Note:</strong> Fragments can be examined using either of the following methods: - the <span class=\'code\'>ogs –xml &lt;fragmentName&gt;</span> command with the fragment name as the <span class=\'code\'>&lt;fragmentname&gt;</span> argument - via <span class=\'code\'>MFragmentManager::getFragmentXML()</span> API interface. The <em>fragmentDumper</em> sample plug-in implements the <span class=\'code\'>dumpFragment</span> command which can also be used to examine fragments using the <span class=\'code\'>-fn</span> option.</p>\n\
</blockquote>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";