var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Geometry Overrides</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Geometry Overrides</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"geometry-overrides\"></a></div>\n\
<p><span class=\'code\'>MPxGeometryOverride</span> is the API entry point for defining high level, draw API agnostic support for custom DAG objects in Viewport 2.0. There is no corresponding class in the legacy interfaces. Viewport 2.0 is primarily structured around retained mode drawing; all geometric data is kept as buffers on the graphics card whenever possible. Any implementation of <span class=\'code\'>MPxGeometryOverride</span> is best thought of as a “data feeder” which gets associated with a particular type of DAG object. In addition to providing data, implementations of <span class=\'code\'>MPxGeometryOverride</span> may also define additional renderables (render items) which are needed to draw the object. The <span class=\'code\'>MPxGeometryOverride</span> interface is very similar to the internal interfaces used to support native Maya objects like polygonal meshes and NURBS surfaces. Thus by using this class, plug-in authors gain access to all internal optimizations such as consolidation and vertex data sharing. In addition, implementations of <span class=\'code\'>MPxGeometryOverride</span> work with internal Maya screen space effects and all supported shaders (including plug-in shaders).</p>\n\
<p><span class=\'code\'>MPxGeometryOverride</span> is not meant for performing custom drawing and thus there is no “draw” method and no access to a draw context. It is also not a plug-in shape. It is simply a class that may be associated with a plug-in shape to describe how to create data buffers for drawing in Viewport 2.0. The actual draw for the DAG object is handled by the shader associated with each render item.</p>\n\
<p>Implementations of <span class=\'code\'>MPxGeometryOverride</span> must be registered with <span class=\'code\'>MDrawRegistry</span> using classification strings. The classification string must begin with “drawdb/geometry” in order to be properly recognized by the system. DAG objects whose classification string satisfies the override classification string are evaluated using the override. Maya creates one instance of the registered <span class=\'code\'>MPxGeometryOverride</span> for each associated DAG object.</p>\n\
<div class=\'figure\'><img src=\'dev_help/images/aa0f8cd7.jpg\' title=\'\'></div>\n\
<p>Figure 38: An <span class=\'code\'>MPxGeometryOverride</span> is associated with a given DAG object by its classification string. Shown is a possible configuration where render items from an internal surface shader as well as custom render items are part of the set of render items that the override would handle.</p>\n\
<p>The interface is divided into several sections or phases. The first phase is <span class=\'code\'>updateDG()</span>, where all attribute data from the associated Maya DAG object must be queried and cached. It is invalid to attempt to query attribute data in the final <span class=\'code\'>populateGeometry()</span> phase, and such behaviour may result in instability. This separation keeps dependency graph evaluation separate from graphics device access in order to facilitate multi-threaded update.</p>\n\
<p>The second phase is <span class=\'code\'>updateRenderItems()</span>. During this phase the override is given a list of pre-created render items, one for each supported shader assigned to the object. The override may disable those render items and/or add additional render items to augment the display of the object. Additional render items need to be associated with shaders using the <span class=\'code\'>MShaderInstance</span> interface. The shaders from each render item determine the total set of geometry requirements that the override needs to fulfill during the <span class=\'code\'>populateGeometry()</span> stage. In <span class=\'code\'>MPxGeometryOverride</span>, additional render items are comparable to the old <span class=\'code\'>MDrawRequest</span> interface.</p>\n\
<p>The third phase is <span class=\'code\'>populateGeometry()</span>. During this phase, the override is given the list of geometry requirements it needs to satisfy, as well as the list of render items that generated those requirements. The override must fill the provided <span class=\'code\'>MGeometry</span> structure with vertex buffers (<span class=\'code\'>MVertexBuffer</span>) to satisfy the requirements. It must also produce an index buffer (<span class=\'code\'>MIndexBuffer</span>) for each render item. The index buffers are stored on the <span class=\'code\'>MGeometry</span> object and must also be directly associated with their corresponding render item using <span class=\'code\'>MRenderItem::associateWithIndexBuffer()</span>.</p>\n\
<div class=\'figure\'><img src=\'dev_help/images/aa0f8cdd.jpg\' title=\'\'></div>\n\
<p>Figure 39: The three phases are used to label the connections between the main constructs which are involved in the update of an <span class=\'code\'>MPxGeometryOverride</span>. The first phase (1.) involves accessing the associated DAG object. The second phase (2.) updates the render items and determines the geometry requirements. The third phase (3.) updates vertex and index buffer data and the indexing association for each render item. In this example, three render items drive the update requirements: one for an associated surface shader and two for custom render items that the override wishes to draw.</p>\n\
<p>The phases of <span class=\'code\'>MPxGeometryOverride</span> are triggered under several different circumstances. The first time a DAG object associated with an <span class=\'code\'>MPxGeometryOverride</span> is created, all phases of the override run. Redraws of the scene do not trigger the update process of the override again unless the associated DAG object has changed. For a DAG object, there are two types of changes that can occur: topology change and data change. Data changes do not trigger a call to <span class=\'code\'>updateRenderItems()</span>; only <span class=\'code\'>updateDG()</span> and <span class=\'code\'>populateGeometry()</span> are called in that case. Topology changes represent a full rebuild and all phases of the override are called. If an override adds additional render items in <span class=\'code\'>updateRenderItems()</span> the first time it runs, those render items are retained across topology updates; therefore it is unnecessary to add them again. A plug-in DAG object can indicate to the Viewport 2.0 change manager that it has changed by calling the static method <span class=\'code\'>MRenderer::setGeometryDrawDirty()</span>. There is a parameter on the method indicating whether the change is a topology or data change.</p>\n\
<p>The Maya SDK example <em>apiMeshShape</em> uses <span class=\'code\'>MPxGeometryOverride</span>. It defines a new class <span class=\'code\'>apiMeshGeometryOverride</span> which provides geometry streams and indexing for shaded draw. It also defines additional render items for wireframe draw which are used for wireframe mode and for displaying the selection wireframe over the shaded object when the object is selected.</p>\n\
<p>The following are some excerpts from <span class=\'code\'>apiMeshGeometryOverride</span> to demonstrate sample behaviour:</p>\n\
<ol>\n\
<li><p>1. Registration with the appropriate classification in <span class=\'code\'>initializePlugin()</span></p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MFnPlugin plugin(...);\n\
// Define some drawDb classification string starting with drawdb/geometry\n\
MString apiMesh::drawDbClassification(&quot;drawdb/geometry/apiMesh&quot;);\n\
\n\
// Register a shape, with a drawdb classification\n\
plugin.registerShape( &quot;apiMesh&quot;, apiMesh::id,\n\
    &amp;apiMesh::creator,\n\
    &amp;apiMesh::initialize,\n\
    &amp;apiMeshUI::creator,\n\
    &amp;apiMesh::drawDbClassification )\n\
\n\
// Register a geometry override with the same drawDB classification\n\
MHWRender::MDrawRegistry::registerGeometryOverrideCreator(\n\
    apiMesh::drawDbClassification,\n\
    apiMesh::drawRegistrantId,\n\
    apiMeshGeometryOverride::Creator);</pre></div></li>\n\
<li><p><span class=\'code\'>updateRenderItems()</span> method creating a “custom” render item to show wireframe.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">// Add a wire frame item if it&#39;s not there\n\
MHWRender::MRenderItem* dormantItem = NULL;\n\
\n\
int index = list.indexOf(\n\
        &quot;apiMeshWire&quot;,\n\
        MHWRender::MGeometry::kLines,\n\
        MHWRender::MGeometry::kWireframe);\n\
\n\
if (index &lt; 0)\n\
{\n\
    MHWRender::MRenderItem* wireItem = new MHWRender::MRenderItem(\n\
            &quot;apiMeshWire&quot;,\n\
            MHWRender::MGeometry::kLines,\n\
            MHWRender::MGeometry::kWireframe,\n\
            false);\n\
    list.append(wireItem);\n\
\n\
    // Use a shader which will provide drawing using a solid color.\n\
    MHWRender::MShaderInstance* shader = shaderMgr-&gt;getStockShader(\n\
        MHWRender::MShaderManager::k3dSolidShader, NULL, NULL );\n\
    if (shader)\n\
    {\n\
        // Set the color on the shader instance using the parameter interface\n\
        static const float theColor[] = {1.0f, 0.0f, 0.0f, 1.0f};\n\
        shader-&gt;setParameter(&quot;solidColor&quot;, theColor);\n\
\n\
        // Assign the shader to the custom render item\n\
        wireItem-&gt;setShader(shader);\n\
    }\n\
}</pre></div></li>\n\
<li><p>Vertex and index buffers are updated, and the index buffer association with render items is performed in <span class=\'code\'>populateGeometry()</span></p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">unsigned int totalVerts = N;\n\
\n\
// Set up three data buffers. One for position, one for normals and one for texture\n\
// coordinates.\n\
MHWRender::MVertexBuffer* positionBuffer = NULL;\n\
float* positions = NULL;\n\
MHWRender::MVertexBuffer* normalBuffer = NULL;\n\
float* normals = NULL;\n\
MHWRender::MVertexBuffer* uvBuffer = NULL;\n\
float* uvs = NULL;\n\
int numUVs = fMeshGeom-&gt;uvcoords.uvcount();\n\
\n\
// Scan through the requirements (descriptor list) one and a time. For each\n\
// descriptor check the semantic. Based on the semantic, update\n\
// the appropriate buffers that this override supports (position, normals and\n\
// texture coordinates).\n\
//\n\
const MHWRender::MVertexBufferDescriptorList&amp; descList = requirements.vertexRequirements();\n\
int numVertexReqs = descList.length();\n\
MHWRender::MVertexBufferDescriptor desc;\n\
for (int reqNum=0; reqNum&lt;numVertexReqs; reqNum++)\n\
{\n\
    if (!descList.getDescriptor(reqNum, desc))\n\
    {\n\
        continue;\n\
    }\n\
\n\
    switch (desc.semantic())\n\
    {\n\
        case MHWRender::MGeometry::kPosition: // Handle position requirement\n\
        {\n\
            if (!positionBuffer)\n\
            {\n\
                positionBuffer = data.createVertexBuffer(desc);\n\
                if (positionBuffer)\n\
                {\n\
                    positions = (float*)positionBuffer-&gt;acquire(totalVerts);\n\
                }\n\
            }\n\
        }\n\
        break;\n\
        case MHWRender::MGeometry::kNormal: // Handle normal requirement\n\
        {\n\
            if (!normalBuffer)\n\
            {\n\
                normalBuffer = data.createVertexBuffer(desc);\n\
                if (normalBuffer)\n\
                {\n\
                    normals = (float*)normalBuffer-&gt;acquire(totalVerts);\n\
                }\n\
            }\n\
        }\n\
        break;\n\
        case MHWRender::MGeometry::kTexture: // Handle texture coordinate requirement\n\
        {\n\
            if (!uvBuffer)\n\
            {\n\
                uvBuffer = data.createVertexBuffer(desc);\n\
                if (uvBuffer)\n\
                {\n\
                    uvs = (float*)uvBuffer-&gt;acquire(totalVerts);\n\
                }\n\
            }\n\
        }\n\
        break;\n\
        default:\n\
            // do nothing for stuff we don&#39;t understand\n\
        break;\n\
    }\n\
}\n\
\n\
// Copy data from shape into data buffers\n\
for (int i=0; i&lt;fMeshGeom-&gt;faceCount; i++)\n\
{\n\
    // ignore degenerate faces\n\
    int numVerts = fMeshGeom-&gt;face_counts[i];\n\
    if (numVerts &gt; 2)\n\
    {\n\
        for (int j=0; j&lt;numVerts; j++)\n\
        {\n\
            if (positions)\n\
            {\n\
                // Fill in position data…\n\
            }\n\
            if (normals)\n\
            {\n\
                // Fill in normal data…\n\
            }\n\
\n\
            if (uvs)\n\
            {\n\
                // Fill in uv data…\n\
            }\n\
\n\
            vid++;\n\
        }\n\
    }\n\
}\n\
\n\
// Commit the updated buffers \n\
if (positions)\n\
    positionBuffer-&gt;commit(positions);\n\
if (normals)\n\
    normalBuffer-&gt;commit(normals);\n\
if (uvs)\n\
    uvBuffer-&gt;commit(uvs);\n\
\n\
// Fill in indexing data for the custom wireframe render item\n\
//\n\
MHWRender::MIndexBuffer* wireIndexBuffer = NULL; \n\
int numItems = renderItems.length();\n\
for (int i=0; i&lt;numItems; i++)\n\
{\n\
    const MHWRender::MRenderItem* item = renderItems.itemAt(i);\n\
    if (!item) continue;\n\
\n\
    // Check for the wireframe render item created updateRenderItems()\n\
    if (item-&gt;name() == &quot;apiMeshWire&quot;) \n\
    {\n\
        // Allocate a wireframe buffer\n\
        if (!wireIndexBuffer)\n\
        {\n\
            wireIndexBuffer = data.createIndexBuffer(MHWRender::MGeometry::kUnsignedInt32);\n\
            if (wireIndexBuffer)\n\
            {\n\
                unsigned int* buffer = (unsigned int*)wireIndexBuffer-&gt;acquire(2*totalVerts);\n\
                if (buffer)\n\
                {\n\
                    // Fill in the buffer here…\n\
\n\
                    // Commit the buffer\n\
                    wireIndexBuffer-&gt;commit(buffer);\n\
                }\n\
            }\n\
        }\n\
\n\
        // Associate same index buffer with either render item\n\
        if (wireIndexBuffer)\n\
        {\n\
            item-&gt;associateWithIndexBuffer(wireIndexBuffer);\n\
        }    \n\
\n\
    }\n\
\n\
    // Handle other render items…\n\
}\n\
</pre></div></li>\n\
</ol>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";