var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Sub-scene Overrides</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Sub-scene Overrides</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"sub-scene-overrides\"></a></div>\n\
<p><span class=\'code\'>MPxSubSceneOverride</span> is the API entry point for defining high-level Viewport 2.0 drawing of plug-in DAG objects that require a large number of drawable entities. Although it can be used with any type of DAG object, <span class=\'code\'>MPxSubSceneOverride</span> is designed to be an efficient and tightly integrated path for supporting “scene-cache” style nodes that manage a large and complex set of objects in a way that is opaque to the rest of Maya. The gpuCache plug-in that is used in Maya’s scene assembly workflow is a good example of this sort of DAG object type.</p>\n\
<p>Implementations of <span class=\'code\'>MPxSubSceneOverride</span> must be registered with <span class=\'code\'>MDrawRegistry</span> using classification strings. The classification string must begin with &quot;drawdb/subscene&quot; in order to be properly recognized by the system. DAG objects with classification strings that satisfy the override classification string are evaluated using the override.</p>\n\
<p><span class=\'code\'>MPxSubSceneOverride</span> is draw-API agnostic. Implementations operate using the <span class=\'code\'>MRenderItem</span> interface and are simply required to manage a collection of <span class=\'code\'>MRenderItem</span> objects to draw all instances of the associated DAG object. One implementation of the override is created for each object in the Maya scene with a classification string that satisfies the classification string used when registering the override. If the associated DAG object is instanced, the override is responsible for drawing all instances.</p>\n\
<p>The collection of render items is stored in an <span class=\'code\'>MSubSceneContainer</span> object. This container is suitable for storing and accessing large numbers of render items (tens of thousands or more) in a high performance manner. Implementations are responsible for assigning a shader and an optional matrix to each render item, and must also provide the geometry for each render item as required by its shader. Geometry may be shared between render items and management of the geometry is up to the implementation.</p>\n\
<p><span class=\'code\'>MPxSubSceneOverride</span> falls in between <span class=\'code\'>MPxDrawOverride</span> and <span class=\'code\'>MPxGeometryOverride</span> with respect to the amount of control given to the implementation. <span class=\'code\'>MPxDrawOverride</span> allows full control over the entire draw of the object, but as a result the implementation is entirely responsible for the draw. <span class=\'code\'>MPxGeometryOverride</span> is relatively simple to work with; but as a result, only limited control of the draw is available. <span class=\'code\'>MPxSubSceneOverride</span> is allowed to fully define all render items, geometry and shaders, providing a high degree of control. However, this definition is abstracted from the hardware draw API; therefore, only one implementation is required to get support for both DirectX and OpenGL. Furthermore, Maya handles the drawing of the render items, and thus the items can participate fully in the Maya rendering pipeline (including screen-space effects such as screen space ambient occlusion, transparency sorting, shadow casting/receiving, and so forth). Finally, render items may specify which draw modes they should draw in (wireframe, shaded, textured, bounding box); therefore, there is no need for the implementation to track these modes manually.</p>\n\
<p><span class=\'code\'>MPxSubSceneOverride</span> has two stages, and the stages are invoked on every frame before drawing begins. In each stage, an instance of <span class=\'code\'>MFrameContext</span> is provided to give some information about the current state of the renderer. The implementation is also free at any time to query any information it needs directly from Maya.</p>\n\
<p>In the <span class=\'code\'>requiresUpdate()</span> stage, implementations of <span class=\'code\'>MPxSubSceneOverride</span> are required to notify Maya if the set of render items for the associated DAG object needs to be updated. If not, then the second stage is skipped. This is an opportunity to avoid heavy processing if the associated node has not changed since the last update.</p>\n\
<p>In the <span class=\'code\'>update()</span> stage, the implementation may add or remove render items from the container and it may modify existing render items. For each render item in the container, the implementation is required to call <span class=\'code\'>MPxSubSceneOverride::setGeometryForRenderItem()</span> to provide the geometry for the render item. Render items without geometry do not draw. The render items may only be modified during the update stage. It is an error to do so at other times and doing so may result in instability.</p>\n\
<p>After the update stage completes, the render items in the container are drawn as long as they are: enabled, have a valid shader, have geometry that fulfills the requirements of the shader, and are not filtered out due to viewport draw modes (that is, wireframe vs. shaded). The render items persist in the container until they are explicitly removed. Therefore, if the data that needs to be drawn is static, then the implementation of <span class=\'code\'>MPxSubSceneOverride</span> could return false on all calls to <span class=\'code\'>requiresUpdate()</span> following the initial update stage.</p>\n\
<p>See the <span class=\'code\'>MRenderItem</span> interface for more information on how a render item may be modified to control its display in Viewport 2.0.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";