var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Example custom transform</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Example custom transform</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"example-custom-transform\"></a></div>\n\
<p>The following are examples of custom transform usage.</p>\n\
<div class=\'section\'><a id=\"custom-transform-attribute-example\"></a><h2 id=\"custom-transform-attribute-example\">Custom transform attribute example</h2></div>\n\
<p>This section describes the <span class=\'code\'>rockingTransform</span> example that is available in the Maya Developer Kit. The <span class=\'code\'>rockingTransform</span> example introduces an attribute that contains a rocking motion along the X-axis. This rocking motion or rotation is stored separately from the regular Rotate attributes but are incorporated into the transformation matrix. Additionally, get/set methods have been added to the class to help you access the new <span class=\'code\'>rockXValue</span> class variable.</p>\n\
<div class=\'section\'><a id=\"implementing-the-proxy-transformation-matrix\"></a><h2 id=\"implementing-the-proxy-transformation-matrix\">Implementing the proxy transformation matrix</h2></div>\n\
<p>The <span class=\'code\'>rockingTransformMatrix</span> class inherits from <span class=\'code\'>MPxTransformationMatrix</span> and defines a number of virtual methods such as <span class=\'code\'>asMatrix()</span>.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">class rockingTransformMatrix : public MPxTransformationMatrix\n\
{\n\
    // A really simple implementation of MPxTransformationMatrix.\n\
    // The methods include:\n\
    // - Two accessor methods for getting and setting the \n\
    // rock\n\
    // - The virtual asMatrix() method which passes the matrix \n\
    // back to Maya when the command &quot;xform -q -ws -m&quot; is invoked\n\
    public:\n\
        rockingTransformMatrix();\n\
        static void *creator();\n\
        \n\
        virtual MMatrix asMatrix() const;\n\
        virtual MMatrix asMatrix(double percent) const;\n\
        virtual MMatrix asRotateMatrix() const;\n\
        \n\
        // Degrees\n\
        double  getRockInX() const;\n\
        void    setRockInX( double rock );\n\
        \n\
        static  MTypeId id;\n\
    protected:      \n\
        typedef MPxTransformationMatrix ParentClass;\n\
        // Degrees\n\
        double rockXValue;\n\
};\n\
\n\
</pre></div><p>The implementation of the <span class=\'code\'>rockingTransformNode</span> involves deriving from <span class=\'code\'>MPxTransform</span> and supporting some required virtuals such as <span class=\'code\'>createTransformationMatrix()</span> and <span class=\'code\'>validateAndSet()</span>. Also, a new <span class=\'code\'>aRockInX</span> attribute has been added to the class.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">class rockingTransformNode : public MPxTransform \n\
{\n\
    // A really simple custom transform.\n\
    public:\n\
        rockingTransformNode();\n\
        rockingTransformNode(MPxTransformationMatrix *);\n\
        virtual ~rockingTransformNode();\n\
        virtual MPxTransformationMatrix *createTransformationMatrix();\n\
\n\
        virtual void postConstructor();\n\
        virtual MStatus validateAndSetValue(\n\
            const MPlug&amp; plug,\n\
            const MDataHandle&amp; handle, \n\
            const MDGContext&amp; context);\n\
\n\
        virtual void resetTransformation (MPxTransformationMatrix *);\n\
        virtual void resetTransformation (const MMatrix &amp;);\n\
\n\
        // Utility for getting the related rock matrix pointer\n\
        rockingTransformMatrix *getRockingTransformMatrix();\n\
\n\
        const char*         className();\n\
        static  void *      creator();\n\
        static MStatus  initialize();\n\
\n\
        static  MTypeId id;\n\
    protected:\n\
        // Degrees\n\
        static  MObject aRockInX;\n\
        double rockXValue;\n\
        typedef MPxTransform ParentClass;\n\
};\n\
\n\
</pre></div><div class=\'section\'><a id=\"initializing-the-plug-in\"></a><h2 id=\"initializing-the-plug-in\">Initializing the Plug-in</h2></div>\n\
<p>You need to register the new transform node and its related transformation matrix with <span class=\'code\'>MFnPlugin</span> when initializing the plug-in. The unique identifiers that are stored in <span class=\'code\'>MTypeIds</span> are required for registering the transform.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus initializePlugin( MObject obj )\n\
{ \n\
    MStatus status;\n\
    MFnPlugin plugin(obj, &quot;Autodesk&quot;, &quot;6.5&quot;, &quot;Any&quot;);\n\
    status = plugin.registerTransform( &quot;rockingTransform&quot;, \n\
        rockingTransformNode::id, &amp;rockingTransformNode::creator, \n\
        &amp;rockingTransformNode::initialize, &amp;rockingTransformMatrix::creator,\n\
        rockingTransformMatrix::id);\n\
    if (!status) {\n\
        status.perror(&quot;registerNode&quot;);\n\
        return status;\n\
    }\n\
    return status;\n\
}\n\
\n\
</pre></div><p>Removing the custom node is done in the <span class=\'code\'>uninitializePlugin()</span> through a call to the <span class=\'code\'>deregisterNode()</span> method of <span class=\'code\'>MFnPlugin</span>.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus uninitializePlugin( MObject obj)\n\
{\n\
    MStatus status;\n\
    MFnPlugin plugin(obj);\n\
    status = plugin.deregisterNode( rockingTransformNode::id );\n\
    if (!status) {\n\
        status.perror(&quot;deregisterNode&quot;);\n\
        return status;\n\
    }\n\
    return status;\n\
}\n\
\n\
</pre></div><div class=\'section\'><a id=\"implementation-of-the-rockingtransform-matrix-class\"></a><h3 id=\"implementation-of-the-rockingtransform-matrix-class\">Implementation of the rockingTransform matrix class</h3></div>\n\
<p>The following examples are very simple implementations of the constructor, creator and get/set methods.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">//\n\
// Matrix constructor. Initialize any\n\
// class variables.\n\
//\n\
rockingTransformMatrix::rockingTransformMatrix()\n\
{\n\
    rockXValue = 0.0;\n\
}\n\
//\n\
// Creator for matrix\n\
//\n\
void *rockingTransformMatrix::creator()\n\
{\n\
    return new rockingTransformMatrix();\n\
}\n\
//\n\
// Utility method for getting the rock\n\
// motion in the X axis\n\
//\n\
double rockingTransformMatrix::getRockInX() const\n\
{\n\
    return rockXValue;\n\
}\n\
//\n\
// Utility method for setting the rcok \n\
// motion in the X axis\n\
//\n\
void rockingTransformMatrix::setRockInX( double rock )\n\
{\n\
    rockXValue = rock;\n\
}\n\
\n\
</pre></div><p>The <span class=\'code\'>asMatrix()</span> method is very important to the custom transformation matrix implementation. This method is what Maya calls when it is requesting the transformation matrix from the custom transform. The implementation of <span class=\'code\'>asMatrix()</span> in the following example calls <span class=\'code\'>ParentClass::asMatrix()</span> to calculate the standard Maya transformation matrix. The rocking motion is then used to add a quaternion rotation to the calculated transformation matrix. This approach allows a custom transform to integrate new attributes into the output of the transformation matrix.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">//\n\
// This method will be used to return information to\n\
// Maya. Use the attributes which are outside of\n\
// the regular transform attributes to build a new\n\
// matrix. This new matrix will be passed back to\n\
// Maya.\n\
//\n\
MMatrix rockingTransformMatrix::asMatrix() const\n\
{\n\
    // Get the current transform matrix\n\
    MMatrix m = ParentClass::asMatrix();\n\
    // Initialize the new matrix we will calculate\n\
    MTransformationMatrix tm( m );\n\
    // Find the current rotation as a quaternion\n\
    MQuaternion quat = rotation();\n\
    // Convert the rocking value in degrees to radians\n\
    DegreeRadianConverter conv;\n\
    double newTheta = conv.degreesToRadians( getRockInX() );\n\
    quat.setToXAxis( newTheta );\n\
    // Apply the rocking rotation to the existing rotation\n\
    tm.addRotationQuaternion( quat.x, quat.y, quat.z, quat.w, MSpace::kTransform );\n\
    // Let Maya know what the matrix should be\n\
    return tm.asMatrix();\n\
}\n\
\n\
</pre></div><p>Note that there is more than one <span class=\'code\'>asMatrix()</span> method. Depending on how the custom transform affects the matrix, all <span class=\'code\'>asMatrix()</span> methods may need to be implemented.</p>\n\
<div class=\'section\'><a id=\"implementing-the-rockingtransformnode\"></a><h3 id=\"implementing-the-rockingtransformnode\">Implementing the rockingTransformNode</h3></div>\n\
<p>As in other proxy nodes implemented using the API, the <span class=\'code\'>initialize()</span> method is used to add new attributes and configure them. In this example, the <span class=\'code\'>aRockInX</span> attribute is added to the node, the attribute is made keyable and is set as <em>affects world space</em>. Also, <span class=\'code\'>mustCallValidateAndSet()</span> is called so that Maya will update correctly as the attribute changes.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">//\n\
//    Node initialize method. We configure node\n\
//    attributes here. Static method so\n\
//    the *this pointer is not available.\n\
//\n\
MStatus rockingTransformNode::initialize()\n\
{\n\
    MFnNumericAttribute numFn;\n\
    aRockInX = numFn.create(&quot;RockInX&quot;, &quot;rockx&quot;,\n\
         MFnNumericData::kDouble, 0.0);\n\
    numFn.setKeyable(true);\n\
    numFn.setAffectsWorldSpace(true);\n\
    addAttribute(aRockInX);\n\
    // This is required so that the validateAndSet method \n\
    // is called\n\
    mustCallValidateAndSet(aRockInX);\n\
    return MS::kSuccess;\n\
}\n\
\n\
</pre></div><p>The standard methods for class implementation are the following:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">//\n\
// Constructor of the transform node\n\
//\n\
rockingTransformNode::rockingTransformNode()\n\
: ParentClass()\n\
{\n\
    rockXValue = 0.0;\n\
}\n\
//\n\
// Constructor of the transform node\n\
//\n\
rockingTransformNode::rockingTransformNode(MPxTransformationMatrix *tm)\n\
: ParentClass(tm)\n\
{\n\
    rockXValue = 0.0;\n\
}\n\
//\n\
// Post constructor method. Has access to *this. Node setup\n\
// operations that do not go into the initialize() method should go\n\
// here.\n\
//\n\
void rockingTransformNode::postConstructor()\n\
{\n\
    // Make sure the parent takes care of anything it needs.\n\
    //\n\
    ParentClass::postConstructor();\n\
    // The baseTransformationMatrix pointer should be setup properly \n\
    // at this point, but just in case, set the value if it is missing.\n\
    //\n\
    if (NULL == baseTransformationMatrix) {\n\
        MGlobal::displayWarning(&quot;NULL baseTransformationMatrix found!&quot;);\n\
        baseTransformationMatrix = new MPxTransformationMatrix();\n\
    }\n\
}\n\
//\n\
// Destructor of the rocking transform\n\
//\n\
rockingTransformNode::~rockingTransformNode()\n\
{\n\
}\n\
//\n\
// Method that returns the new transformation matrix\n\
//\n\
MPxTransformationMatrix *rockingTransformNode::createTransformationMatrix()\n\
{\n\
    return new rockingTransformMatrix();\n\
}\n\
//\n\
// Method that returns a new transform node\n\
//\n\
void *rockingTransformNode::creator()\n\
{\n\
    return new rockingTransformNode();\n\
}\n\
\n\
</pre></div><p>The <span class=\'code\'>validateAndSetValue()</span> virtual can be used to make sure that attribute input is correct. For example, it should confirm whether an attribute is locked or clamped. In this simple case, the method can be implemented to ignore locking and clamping. This approach has been taken in the following implementation:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus rockingTransformNode::validateAndSetValue(\n\
const MPlug&amp; plug,\n\
    const MDataHandle&amp; handle,\n\
    const MDGContext&amp; context)\n\
{\n\
    MStatus status = MS::kSuccess;\n\
    // Make sure that there is something interesting to process.\n\
    //\n\
    if (plug.isNull())\n\
        return MS::kFailure;\n\
    MDataBlock block = forceCache(*(MDGContext *)&amp;context);\n\
    MDataHandle blockHandle = block.outputValue(plug, &amp;status);\n\
    ReturnOnError(status);\n\
    \n\
    if ( plug == aRockInX )\n\
    {\n\
        // Update our new rock in x value\n\
        double rockInX = handle.asDouble();\n\
        blockHandle.set(rockInX);\n\
        rockXValue = rockInX;\n\
        \n\
        // Update the custom transformation matrix to the\n\
        // right rock value. \n\
        rockingTransformMatrix *ltm = getRockingTransformMatrix();\n\
        if (ltm)\n\
            ltm-&gt;setRockInX(rockXValue);\n\
        else \n\
            MGlobal::displayError(&quot;Failed to get rock transform matrix&quot;);\n\
            \n\
        blockHandle.setClean();\n\
        \n\
        // Mark the matrix as dirty so that DG information\n\
        // will update.\n\
        dirtyMatrix();        \n\
    }\n\
    \n\
    // Allow processing for other attributes\n\
    return ParentClass::validateAndSetValue(plug, handle, context);\n\
}\n\
\n\
</pre></div><p>There are a number of virtuals that the <span class=\'code\'>MPxTransform</span> node provides for testing locking and clamping. For example, to handle locking and clamping correctly with rotation, the methods <span class=\'code\'>checkAndSetRotation()</span>, <span class=\'code\'>applyRotationLimits()</span> and <span class=\'code\'>applyRotationLocks()</span> can be implemented. A similar set of methods exists for other attributes. The <span class=\'code\'>rockingTransformCheck</span> example in the Maya Developer Kit demonstrates these principles for rotation.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";